const utils = require('@iobroker/adapter-core'); // Get common adapter utils
const { exec } = require('child_process');

class HoymilesWifi extends utils.Adapter {
    constructor(options) {
        super({
            ...options,
            name: 'hoymiles-wifi',
        });

        this.on('ready', this.onReady.bind(this));
        this.on('unload', this.onUnload.bind(this));
    }

    async onReady() {
        // Create the "info.connection" state
        await this.setObjectNotExistsAsync('info.connection', {
            type: 'state',
            common: {
                name: 'Connection',
                type: 'boolean',
                role: 'indicator.connected',
                read: true,
                write: true,
            },
            native: {},
        });

        // Set the initial connection state to true
        await this.setStateAsync('info.connection', { val: true, ack: true });

        // Create the "Online" state
        await this.setObjectNotExistsAsync('Online', {
            type: 'state',
            common: {
                name: 'Online',
                type: 'boolean',
                role: 'indicator.connected',
                read: true,
                write: false,
            },
            native: {},
        });

        // Schedule regular data fetching
        this.fetchData();
        this.scheduleFetch();
    }

    onUnload(callback) {
        try {
            clearTimeout(this.fetchTimeout);
            callback();
        } catch (e) {
            callback();
        }
    }

    async fetchData() {
        const host = this.config.host || '192.168.178.11'; // Default host if not set
        const commands = [
            'get-real-data-new',
            'get-real-data',
            'get-config',
            'network-info',
            'app-information-data',
            'app-get-hist-power',
            'get-information-data',
            'get-version-info',
            'heartbeat',
            'get-alarm-list'
        ];

        let online = true;

        for (const command of commands) {
            try {
                const result = await this.executeCommand(command, host);
                await this.storeData(command, result);
            } catch (error) {
                this.log.debug(`Error executing command ${command}: ${error.message}`);
                online = false;
            }
        }

        await this.setStateAsync('Online', { val: online, ack: true });
        await this.setStateAsync('info.connection', { val: online, ack: true });
    }

    async executeCommandWithRetry(command, host, retries = 3, delay = 5000) {
        for (let attempt = 1; attempt <= retries; attempt++) {
            try {
                const result = await this.executeCommand(command, host);
                await this.storeData(command, result);
                return; // Wenn erfolgreich, beende die Schleife
            } catch (error) {
                this.log.debug(`Error executing command ${command}: ${error.message}`);
                if (attempt < retries) {
                    this.log.info(`Retrying command ${command} in ${delay / 1000} seconds...`);
                    await new Promise(resolve => setTimeout(resolve, delay));
                } else {
                    this.log.error(`Failed to execute command ${command} after ${retries} attempts.`);
                }
            }
        }
    }

    scheduleFetch() {
        const interval = parseFloat(this.config.pollInterval) || 60;
        this.fetchTimeout = setTimeout(() => {
            this.fetchData();
            this.scheduleFetch();
        }, interval * 1000);
    }

    executeCommand(command, host) {
        return new Promise((resolve, reject) => {
            this.log.info(`Executing command: ${command}`);
            exec(`hoymiles-wifi --host ${host} --as-json ${command}`, (error, stdout, stderr) => {
                if (error) {
                    return reject(error);
                }
                if (!this.isValidJson(stdout)) {
                    return reject(new Error('Invalid JSON response'));
                }
                try {
                    const data = JSON.parse(stdout);
                    resolve(data);
                } catch (parseError) {
                    reject(parseError);
                }
            });
        });
    }

    isValidJson(data) {
        try {
            JSON.parse(data);
            return true;
        } catch {
            return false;
        }
    }

    async storeData(command, data) {
        const statePath = `${this.namespace}.${command.replace(/-/g, '_')}`;
        try {
            // Check if the main data is an object and process it
            if (typeof data === 'object' && data !== null) {
                for (const [key, value] of Object.entries(data)) {
                    await this.processData(`${statePath}.${key}`, key, value);
                }
            } else {
                await this.processData(statePath, command, data);
            }
        } catch (error) {
            this.log.error(`Error storing data for command ${command}: ${error.message}`);
        }
    }

    async processData(path, name, value) {
        if (Array.isArray(value)) {
            // Create a folder for the array
            await this.setObjectNotExistsAsync(path, {
                type: 'channel',
                common: {
                    name: name,
                },
                native: {},
            });
            // Process each element of the array
            for (let i = 0; i < value.length; i++) {
                await this.processData(`${path}.${i}`, `${name}[${i}]`, value[i]);
            }
        } else if (typeof value === 'object' && value !== null) {
            // Create a folder for the object
            await this.setObjectNotExistsAsync(path, {
                type: 'channel',
                common: {
                    name: name,
                },
                native: {},
            });
            // Process each property of the object
            for (const [key, subValue] of Object.entries(value)) {
                await this.processData(`${path}.${key}`, key, subValue);
            }
        } else {
            // Create and store the state
            let type = typeof value;
            let role = 'value';

            // Check if the value is a timestamp
            if (type === 'number' && this.isTimestamp(value)) {
                value = this.convertTimestamp(value);
                type = 'string';
                role = 'date';
            }

            await this.setObjectNotExistsAsync(path, {
                type: 'state',
                common: {
                    name: name,
                    type: type,
                    role: role,
                    read: true,
                    write: false,
                },
                native: {},
            });
            await this.setStateAsync(path, { val: value, ack: true });
        }
    }

    isTimestamp(value) {
        // Check if the number is a reasonable timestamp
        const date = new Date(value * 1000);
        return date.getFullYear() > 2000 && date.getFullYear() < 2100;
    }

    convertTimestamp(value) {
        // Convert the timestamp to a readable date string
        const date = new Date(value * 1000);
        return date.toISOString();
    }
}

if (module.parent) {
    module.exports = (options) => new HoymilesWifi(options);
} else {
    new HoymilesWifi();
}
